# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.media_uri import MediaUri
from elisa.plugins.poblesec.link import Link
from twisted.internet import defer
from elisa.core.utils.i18n import install_translation
from elisa.core.common import application

_ = install_translation('poblesec')

def music_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music_menu"
    link.label = _("Music")
    link.icon = "elisa.plugins.poblesec.music_section"
    controller.model.append(link)

    return defer.succeed(None)

def albums_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music/albums"
    link.label = _("Albums")
    link.icon = "elisa.plugins.poblesec.album"
    # FIXME: temporarily deactivated
#    controller.model.append(link)

    return defer.succeed(None)

def artists_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music/artists"
    link.label = _("Artists")
    link.icon = "elisa.plugins.poblesec.artist"
    # FIXME: temporarily deactivated
#    controller.model.append(link)

    return defer.succeed(None)

def filesystem_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music/directory_list"
    link.label = _("Folders")
    link.icon = "elisa.plugins.poblesec.music_folders"
    link.controller_args = {'media_type': 'music'}
    controller.model.append(link)

    return defer.succeed(None)

def internet_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music/internet"
    link.label = _("Internet")
    link.icon = "elisa.plugins.poblesec.internet"
    controller.model.append(link)

    return defer.succeed(None)

def network_decorator(controller):
    link = Link()
    link.controller_path = "/poblesec/music/network"
    link.label = _("Home network")
    link.icon = "elisa.plugins.poblesec.network"
    link.controller_args = {'media_type': 'music'}
    controller.model.append(link)

    return defer.succeed(None)

def hardware_decorator(controller):
    uri = MediaUri("volumes://?filter=removable")
    model, dfr = application.resource_manager.get(uri, None)

    def do_list_hardware(model):
        devices = sorted(model.volumes, key=lambda i: i.label.lower())

        for dev in devices:
            link = Link()
            if not dev.mount_point:
                continue
            if dev.protocol == 'file':
                link.controller_path = "/poblesec/music/filesystem"
                link.icon = "elisa.plugins.poblesec.removable_device"
                link.controller_args = {'uri': dev.mount_point, 'media_type': 'audio'}
            elif dev.protocol == 'ipod':
                link.controller_path = "/poblesec/music/ipod"
                link.icon = "elisa.plugins.poblesec.ipod"
                uri = MediaUri(dev.mount_point)
                uri.set_param('action', 'tracks')
                uri.set_param('search', 'artist')
                link.controller_args = {'uri': uri}
            else:
                # don't show
                continue

            link.label = dev.label
            controller.model.append(link)

        return model

    def do_list_hardware_error(err, *args):
        return err

    dfr.addCallback(do_list_hardware)
    dfr.addErrback(do_list_hardware_error)

    return dfr
